﻿
//_______________________________________________

//L E G O   N X T B E E  O R   B O E - B O T   R E M O T E   C O N T R O L
//                                Ron Kessler
//                                12/16/2011
//'_______________________________________________

//Purpose: This controls a Lego NXT or a Boe-Bot via XBEE-PRO radios
//Radios must be connected to port 4 on NXT and that port must use high speed parameter. This is set using
//the DIGI X-CTU program. Be sure to write this info to both your XBEE's.
//Form properties: KeyPreview = True

//Updated 3/9/12

//12/17/2011  finished debugging
//3/9/2012  Added code to run the Boe-Bot. User can select which bot on UI
//  see updates in "Make it so" region

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO.Ports;
using System.Threading;




namespace XBEE_NXT_Remote_Control
{
     public partial class frmMain : Form
    {
        #region "Constructor"
        public frmMain()
        {
            InitializeComponent();
            lblStatus.Text = General.DisconnectedMsg;
            btnConnect.Enabled = false;
            grpMotors.Enabled = false;
        }
        #endregion

        #region "Load and Initialize combo box for Com Ports & Choose which BOT"

        private void frmMain_Load(object sender, EventArgs e)
        {
            //---show all available Com ports

            foreach (string sp in SerialPort.GetPortNames())
            {
                cboPorts.Items.Add(sp); 
            }
                
            cboPorts.SelectedIndex =-1;
        }
       

        private void cboPorts_SelectedIndexChanged(object sender, EventArgs e)
        {
            try
            {
                General.whichComPort = cboPorts.SelectedItem.ToString();
                if (General.whichComPort.Substring(0,3) == "COM")
                    btnConnect.Enabled = true;
                else
                    btnConnect.Enabled = false;
            }
            catch (Exception ex)
            {
                lblStatus.Text = "Cannot access that port." + ex.Message.ToString();

            }
        }

        private void whichBot_CheckedChanged(object sender, EventArgs e)
        {
            if (rdoNXT.Checked)
                General.whichBot = "NXT";
            else
                General.whichBot = "BOE";

        }

        #endregion

        #region "Make Com Connection"
        private void btnConnect_Click(object sender, EventArgs e)
        {
             try
             {
           
                General.RS232.PortName = General.whichComPort.Trim();
                General.RS232.BaudRate = 9600;
                General.RS232.Parity = Parity.None;
                General.RS232.StopBits = StopBits.One;
                General.RS232.DataBits = 8;
                General.RS232.Open();
            
                lblStatus.Text = General.ConnectedMsg;
                btnConnect.Enabled = false;
                grpMotors.Enabled = true;
                cboPorts.Focus();         //keeps buttons from handling my arrow keydown events
             }
        catch (Exception ex)
             {
                MessageBox.Show("Unable to connect." + ex.Message.ToString(), "System Message",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
                lblStatus.Text = General.DisconnectedMsg;
                btnConnect.Enabled = true;
                grpMotors.Enabled = false;
            }
        
        }
        #endregion

        #region "Idle & Disconnect"
        private void btnDisconnect_Click(object sender, EventArgs e)
        {
            Disconnect();
        }

        private void Disconnect()
        { 
            //---clean up
            lblStatus.Text = General.DisconnectedMsg;
            grpMotors.Enabled = false;
            General.RS232.Close();
           
        }
      

        private void frmMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            Disconnect();
        }
       

        private void btnExit_Click(object sender, EventArgs e)
        {
            Disconnect();
            this.Close();

        }
        #endregion

        #region "Make it so! Direction Buttons & Arrow keys"
        private void Direction_Controls(object sender, EventArgs e)
        {
            Button whichButton = (Button)sender;

            //---create a code for the boe-bot
            int BOECode;

            switch (whichButton.Name)
            {
                case "btnForward":
                   
                    General.asciiCode = "102";
                    BOECode = 102;
                    break;
                case "btnReverse":
                   
                    General.asciiCode = "98";
                    BOECode = 98;
                    break;
                case "btnRight":
                    BOECode = 114;
                    General.asciiCode = "114";
                    break;
                case "btnLeft":
                    BOECode = 108;
                    General.asciiCode="108";
                    break;
                case "btnIdle":
                    BOECode = 115;
                    General.asciiCode = "115";
                    break;
                default:
                    BOECode = 0;
                    General.asciiCode = "0";
                    break;
                    
            }
            //---now move the bot

            if (General.whichBot == "BOE")
            {
                
                    byte[] myCmd = { (byte)BOECode, 13, 10 };
                    General.RS232.Write(myCmd, 0, myCmd.Length);
                
            }
            else

                if (General.asciiCode != "0")
                {
                    General.RS232.WriteLine(General.asciiCode);
                    Thread.Sleep(150);                                          //wait for data mode
                    General.RS232.WriteLine(General.asciiCode);
                }
            cboPorts.Focus();
        }

        private void frmMain_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Left:
                    e.Handled=true;
                    btnLeft.PerformClick();
                    break;
                case Keys.Right:
                    e.Handled=true;
                    btnRight.PerformClick();
                    break;
                case Keys.Up:
                    e.Handled=true;
                    btnForward.PerformClick();
                    break;
                case Keys.Down:
                    e.Handled=true;
                    btnReverse.PerformClick();
                    break;
                case Keys.End:
                    e.Handled=true;
                    btnIdle.PerformClick();
                    break;
            }
            cboPorts.Focus();
        }

        #endregion

     

    }
}
